# SQL Command Definitions:  MySQL / conference info database

# Various tables to keep track of conferences, items, and responses.  Right
# now very little is in the database.  More will go in as the CDB option
# evolves.


# Create the index database

cdb_create()
{
    # Item table.  This replaces and extends Picospan sum files.  Each item
    # has a unique ID number (never shown to users).  The author alias and
    # title may be truncated versions of the ones stored in the item files.
    # Start_date is the date it was created, resp_date is the date of the
    # most recent response.  It has been split into two tables for MySQL
    # performance reasons - the bt_item table has static row lengths but
    # includes all the info needed to do a "READ NEW".  The dynamic fields
    # are in the bt_item_info table.

    -DROP TABLE bt_item;
    -DROP TABLE bt_item_info;
    CREATE TABLE bt_item (
	   item_id		CHAR(24) PRIMARY KEY,
	   start_date	        &type_time NOT NULL,
	   post_date	        &type_time NOT NULL,
	   frozen               TINYINT UNSIGNED NOT NULL,
	   retired              TINYINT UNSIGNED NOT NULL,
	   n_resp		MEDIUMINT UNSIGNED NOT NULL
    );

    CREATE TABLE bt_item_info (
	   item_id		CHAR(24) PRIMARY KEY,
	   author_id		&type_login NOT NULL,
	   author_uid		&type_uid NOT NULL,
	   author_alias		&type_fname NOT NULL,
	   title		VARCHAR(128) NOT NULL,
           keywords		TINYTEXT NOT NULL,
           summary		TINYTEXT NOT NULL,
	   FULLTEXT INDEX (title, keywords, summary)
    ) TYPE=MyISAM;

    # Link table.  Tell which items are in which conferences.
    -DROP TABLE bt_link;
    CREATE TABLE bt_link (
	   item_id		CHAR(24) NOT NULL,
	   conf_id		MEDIUMINT UNSIGNED NOT NULL,
	   item_no		MEDIUMINT UNSIGNED NOT NULL,
	   link_date		&type_time NOT NULL,
	   UNIQUE INDEX(item_id),
	   UNIQUE INDEX (conf_id, item_no)
    );

    # Conference Table.  
    -DROP TABLE bt_conf;
    CREATE TABLE bt_conf (
	   conf_id		MEDIUMINT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
	   part_name		VARCHAR(32),
	   UNIQUE INDEX (part_name)
    );

    # Response table.  This replaces and extends Backtalk's old index files.
    # Response id is a globally unique identifer for responses.
    # Main use is to record offsets of all responses in the item file, but
    # we also duplicate some info that is also in the item file.  Status is
    # 0=normal, 1=hidden, 2=erased.
#    -DROP TABLE bt_resp;
#    CREATE TABLE bt_resp (
#	   resp_id		MEDIUMINT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
#	   item_id	        CHAR(24) NOT NULL,
#	   resp_no              MEDIUMINT UNSIGNED,
#	   parent		MEDIUMINT UNSIGNED,
#	   author_id		&type_login,
#	   author_alias		&type_fname,
#	   resp_date            DATETIME,
#	   edit_date            DATETIME,
#	   status               TINYINT UNSIGNED,
#	   offset		INT UNSIGNED,
#          UNIQUE INDEX (item_id,resp_no);
#    ) TYPE=MyISAM;

}

# Create a new conference table entry, and return it's unique ID number.
cdb_new_conf($p)
{
    INSERT INTO bt_conf (part_name) VALUES (LEFT($'p,32));
    SELECT LAST_INSERT_ID();
}


# Given a participation file name, return a conference ID number
cdb_conf_id($p)
{
    SELECT conf_id FROM bt_conf WHERE part_name=LEFT($'p,32);
}


# Delete a conference table entry
cdb_del_conf($p)
{
    DELETE FROM bt_conf WHERE part_name=LEFT($'p,32);
}

# Create a new item table entry.
cdb_new_item($i,$d,$p,$f,$r,$n,$l,$u,$a,$t,$k,$s)
{
    INSERT INTO bt_item
	(item_id, start_date, post_date, frozen, retired, n_resp)
      VALUES ($'i, $d, $p, $f, $r, $n);
    INSERT INTO bt_item_info
       (item_id, author_id, author_uid, author_alias, title, keywords, summary)
      VALUES ($'i,$'l, $u, $'a, $'t, $'k, $'s);
}

# Delete an item table entry
cdb_del_item($i)
{
    DELETE FROM bt_item WHERE item_id=$'i;
    DELETE FROM bt_item_info WHERE item_id=$'i;
}


# Get item info.
# 1:  author's login ID
# 2:  author's uid number
# 3:  author's fullname or alias
# 4:  title for this item
# 5:  keyword list for this item
# 6:  summary text for this item

cdb_item_info($i)
{
    SELECT author_id, author_uid, author_alias, title, keywords, summary
	FROM bt_item_info
	WHERE item_id=$'i;
}


# Update summary/keywords

cdb_update_sum_key($i,$s,$k)
{
    UPDATE bt_item_info SET summary=$'s, keywords=$'k WHERE item_id=$'i;
}


# Get related items given a query string sorted by decreasing relevance.  If
# the conference id is 0 it searchs the entire conferencing system.  A limit
# on the maximum responses can be given.
# 1:  score
# 2:  itemid
# 3:  item number in conference (only if conf arg was given)

cdb_related_items($s,$c,$n)
{
    SELECT MATCH title, keywords, summary AGAINST ($'s),
	   bt_item_info.item_id
	   $[c , item_no]
	FROM $[c bt_link,] bt_item_info
	WHERE MATCH title, keywords, summary AGAINST ($'s)
           $[c AND bt_item_info.item_id=bt_link.item_id AND conf_id=$c]
        $[n LIMIT $n];
}

# Set new item title

cdb_item_title($i,$t)
{
    UPDATE bt_item_info SET title=$'t WHERE item_id=$'i;
}


# Create a new link table entry, given the item's unique ID, the conference
# ID number, the item number, and the link date.

cdb_new_link($i,$c,$n,$d)
{
    INSERT INTO bt_link (item_id, conf_id, item_no, link_date)
      VALUES ($'i, $c, $n, $d);
}

# Delete a link table entry.  Return the item id.

cdb_del_link($i,$c)
{
    DELETE FROM bt_link WHERE item_id=$'i AND conf_id=$c;
}

# Return a count of the number of links to a given item
cdb_n_links($i)
{
    SELECT count(*) FROM bt_link WHERE item_id=$'i;
}

# For each conference to which this item is linked, return
#  1: the conferenence id
#  2: item number in that conference.
#  3: date linked into that conference.

cdb_links($i,$n)
{
    SELECT conf_id, item_no, link_date
       FROM bt_link
       WHERE item_id=$'i
       ORDER BY link_date
       $[n LIMIT $n];
}

# Given a conference ID and item number, find the unique ID and link date
# for the item.
# 1:  item ID
# 2:  link date

cdb_link_info($c,$n)
{
    SELECT item_id, link_date FROM bt_link WHERE conf_id=$c AND item_no=$n;
}
